@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2016 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2018 The gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,14 +18,13 @@
 #
 
 # This metadata block gets parsed by setup.py - use single quotes only
-__tagline__   = 'Media aggregator and podcast client'
-__author__    = 'Thomas Perl <thp@gpodder.org>'
-__version__   = '3.9.1'
-__date__      = '2016-08-31'
-__relname__   = 'My BLT Drive Just Went AWOL'
-__copyright__ = '© 2005-2016 Thomas Perl and the gPodder Team'
-__license__   = 'GNU General Public License, version 3 or later'
-__url__       = 'http://gpodder.org/'
+__tagline__ = 'Media aggregator and podcast client'
+__author__ = 'Thomas Perl <thp@gpodder.org>'
+__version__ = '3.10.7'
+__date__ = '2019-02-01'
+__copyright__ = '© 2005-2019 The gPodder Team'
+__license__ = 'GNU General Public License, version 3 or later'
+__url__ = 'http://gpodder.org/'
 
 __version_info__ = tuple(int(x) for x in __version__.split('.'))
 
@@ -35,47 +34,49 @@ import platform
 import gettext
 import locale
 
+from gpodder.build_info import BUILD_TYPE
+
 # Check if real hard dependencies are available
 try:
-    import feedparser
+    import podcastparser
 except ImportError:
-    print """
-  Error: Module "feedparser" (python-feedparser) not found.
-         The feedparser module can be downloaded from
-         http://code.google.com/p/feedparser/
+    print("""
+  Error: Module "podcastparser" (python-podcastparser) not found.
+         The podcastparser module can be downloaded from
+         http://gpodder.org/podcastparser/
 
   From a source checkout, you can download local copies of all
   CLI dependencies for debugging (will be placed into "src/"):
 
-      python tools/localdepends.py
-"""
+      python3 tools/localdepends.py
+""")
     sys.exit(1)
-del feedparser
+del podcastparser
 
 try:
     import mygpoclient
 except ImportError:
-    print """
+    print("""
   Error: Module "mygpoclient" (python-mygpoclient) not found.
          The mygpoclient module can be downloaded from
-         http://thp.io/2010/mygpoclient/
+         http://gpodder.org/mygpoclient/
 
   From a source checkout, you can download local copies of all
   CLI dependencies for debugging (will be placed into "src/"):
 
-      python tools/localdepends.py
-"""
+      python3 tools/localdepends.py
+""")
     sys.exit(1)
 del mygpoclient
 
 try:
     import sqlite3
 except ImportError:
-    print """
+    print("""
   Error: Module "sqlite3" not found.
          Build Python with SQLite 3 support or get it from
          http://code.google.com/p/pysqlite/
-"""
+""")
     sys.exit(1)
 del sqlite3
 
@@ -83,6 +84,7 @@ del sqlite3
 # The User-Agent string for downloads
 user_agent = 'gPodder/%s (+%s) %s/%s' % (__version__, __url__, platform.system(), platform.release())
 
+
 # Are we running in GUI or console mode?
 class UI(object):
     def __init__(self):
@@ -104,34 +106,27 @@ dbus_session_bus = None
 ui.win32 = (platform.system() == 'Windows')
 # Set "osx" to True if we are on Mac OS X
 ui.osx = (platform.system() == 'Darwin')
+# We assume it's a freedesktop.org system if it's not Windows or OS X
+ui.freedesktop = not ui.win32 and not ui.osx
 
 # i18n setup (will result in "gettext" to be available)
 # Use   _ = gpodder.gettext   in modules to enable string translations
 textdomain = 'gpodder'
 locale_dir = gettext.bindtextdomain(textdomain)
-t = gettext.translation(textdomain, locale_dir, fallback=True)
-
-try:
-    # Python 2
-    gettext = t.ugettext
-    ngettext = t.ungettext
-except AttributeError:
-    # Python 3
-    gettext = t.gettext
-    ngettext = t.ngettext
 
 if ui.win32:
-    try:
-        # Workaround for bug 650
-        from gtk.glade import bindtextdomain
-        bindtextdomain(textdomain, locale_dir)
-        del bindtextdomain
-    except:
-        # Ignore for missing glade module
-        pass
+    # this must be done prior to gettext.translation to set the locale (see #484)
+    from gpodder.utilwin32locale import install
+    install(textdomain, locale_dir)
+
+t = gettext.translation(textdomain, locale_dir, fallback=True)
+
+gettext = t.gettext
+ngettext = t.ngettext
+
 del t
 
-# Set up textdomain for gtk.Builder (this accesses the C library functions)
+# Set up textdomain for Gtk.Builder (this accesses the C library functions)
 if hasattr(locale, 'bindtextdomain'):
     locale.bindtextdomain(textdomain, locale_dir)
 
@@ -146,13 +141,12 @@ del SOCKET_TIMEOUT
 
 # Variables reserved for GUI-specific use (will be set accordingly)
 ui_folders = []
-credits_file = None
 icon_file = None
 images_folder = None
 user_extensions = None
 
 # Episode states used in the database
-STATE_NORMAL, STATE_DOWNLOADED, STATE_DELETED = range(3)
+STATE_NORMAL, STATE_DOWNLOADED, STATE_DELETED = list(range(3))
 
 # Paths (gPodder's home folder, config, db, download and data prefix)
 home = None
@@ -163,6 +157,7 @@ prefix = None
 
 ENV_HOME, ENV_DOWNLOADS = 'GPODDER_HOME', 'GPODDER_DOWNLOAD_DIR'
 
+
 # Function to set a new gPodder home folder
 def set_home(new_home):
     global home, config_file, database_file, downloads
@@ -173,10 +168,33 @@ def set_home(new_home):
     if ENV_DOWNLOADS not in os.environ:
         downloads = os.path.join(home, 'Downloads')
 
+
 def fixup_home(old_home):
-    if ui.osx:
-        new_home = os.path.expanduser(os.path.join('~', 'Library',
-            'Application Support', 'gPodder'))
+    if ui.osx or ui.win32:
+        if ui.osx:
+            new_home = os.path.expanduser(os.path.join('~', 'Library',
+                'Application Support', 'gPodder'))
+        elif BUILD_TYPE == 'windows-portable':
+            new_home = os.path.normpath(os.path.join(os.path.dirname(sys.executable), "..", "..", "config"))
+            old_home = new_home  # force to config directory
+            print("D: windows-portable build; forcing home to config directory %s" % new_home, file=sys.stderr)
+        else:  # ui.win32, not portable build
+            from gpodder.utilwin32ctypes import get_documents_folder, get_reg_current_user_string_value
+            try:
+                # from old launcher, see
+                # https://github.com/gpodder/gpodder/blob/old/gtk2/tools/win32-launcher/folderselector.c
+                new_home = get_reg_current_user_string_value("Software\\gpodder.org\\gPodder", "GPODDER_HOME")
+                print("D: windows build; registry home = %s" % new_home, file=sys.stderr)
+            except Exception as e:
+                print("E: can't get GPODDER_HOME from registry: %s" % e, file=sys.stderr)
+                new_home = None
+            if new_home is None:
+                try:
+                    new_home = os.path.join(get_documents_folder(), "gPodder")
+                    print("D: windows build; documents home = %s" % new_home, file=sys.stderr)
+                except Exception as e:
+                    print("E: can't get user's Documents folder: %s" % e, file=sys.stderr)
+                    new_home = old_home
 
         # Users who do not have the old home directory, or who have it but also
         # have the new home directory (to cater to situations where the user
@@ -187,25 +205,27 @@ def fixup_home(old_home):
 
     return old_home
 
+
 # Default locations for configuration and data files
 default_home = os.path.expanduser(os.path.join('~', 'gPodder'))
 default_home = fixup_home(default_home)
 set_home(os.environ.get(ENV_HOME, default_home))
 
 if home != default_home:
-    print >>sys.stderr, 'Storing data in', home, '(GPODDER_HOME is set)'
+    print('Storing data in', home, '(GPODDER_HOME is set)', file=sys.stderr)
 
 if ENV_DOWNLOADS in os.environ:
     # Allow to relocate the downloads folder (pull request 4, bug 466)
     downloads = os.environ[ENV_DOWNLOADS]
-    print >>sys.stderr, 'Storing downloads in %s (%s is set)' % (downloads,
-            ENV_DOWNLOADS)
+    print('Storing downloads in %s (%s is set)' % (downloads,
+            ENV_DOWNLOADS), file=sys.stderr)
 
 # Plugins to load by default
 DEFAULT_PLUGINS = [
     'gpodder.plugins.soundcloud',
 ]
 
+
 def load_plugins():
     """Load (non-essential) plugin modules
 
@@ -220,5 +240,5 @@ def load_plugins():
     for plugin in PLUGINS:
         try:
             __import__(plugin)
-        except Exception, e:
-            print >>sys.stderr, 'Cannot load plugin: %s (%s)' % (plugin, e)
+        except Exception as e:
+            print('Cannot load plugin: %s (%s)' % (plugin, e), file=sys.stderr)
